from functools import wraps
from time import sleep

import requests

from .exceptions import WaitForProcess

def retry_on_processing(f):
    @wraps(f)
    def wrapper(*args, **kwargs):
        c = 0
        r = None
        while c < 5:
            try:
                return f(*args, **kwargs)
            except WaitForProcess:
                c += 1
                sleep(1)
        raise requests.exceptions.RetryError("Too many attempts")
    return wrapper


def throw_for(code, exc):
    def wrapped(f):
        @wraps(f)
        def wrapper(*args, **kwargs):
            r = f(*args, **kwargs)
            if r.status_code == code:
                raise exc
            return r
        return wrapper
    return wrapped


def returns_json(f):
    @wraps(f)
    def wrapper(*args, **kwargs):
        r = f(*args, **kwargs)
        return r.json()
    return wrapper

def returns_gen(f):
    @wraps(f)
    def wrapper(*args, gen=False, **kwargs):
        g = f(*args, **kwargs)
        return g if gen else list(g)
    return wrapper



import requests
from urllib.parse import urljoin


class BaseAPI(object):
    @property
    def session(self):
        return self._session

    def get(self, *args, **kwargs):
        return self.session.get(*args, **kwargs)

    def post(self, *args, **kwargs):
        return self.session.post(*args, **kwargs)

    def put(self, *args, **kwargs):
        return self.session.put(*args, **kwargs)

    def delete(self, *args, **kwargs):
        return self.session.delete(*args, **kwargs)

    def logout(self):
        if not getattr(self, '_session', None):
            return False
        self._session.close()
        self._session = requests.Session()
        self._session.verify = self.verify_ssl
        return True


class VSpotBase(BaseAPI):
    def __init__(self, base_url, apikey=None, verify_ssl=True):
        self.base_url = base_url
        self.apikey = apikey
        self.verify_ssl = verify_ssl
        self._session = requests.Session()
        self._session.verify = self.verify_ssl
        self._set_auth()

    def endpoint(self, path, *args, version='1'):
        url = urljoin(self.base_url, '/api/v%s/' % version)
        return urljoin(url, path.format(*args))

    def get_file(self, uri):
        '''
            To get files like map images the user must be authenticated
            this function gets the file through the authenticated session
            returns tuple (mime type, file bytes)
            as in ('image/png', b'\\x89PNG\\r\\n\\x...')

            # TODO: iter chunks for big files
        '''
        with self.get(uri, stream=True) as r:
            return r.headers.get('Content-Type'), r.content

    def link_gen(self, response, unwind=True):
        oresp = response

        d = oresp.json()
        if isinstance(d, list) and unwind:
            for x in d:
                yield x
        else:
            yield d
        while 'url' in oresp.links.get('next', ''):
            resp = self.get(oresp.links['next']['url'])
            del oresp
            d = resp.json()
            if isinstance(d, list) and unwind:
                for x in d:
                    yield x
            else:
                yield d
            oresp = resp
            del x, d, resp


    def _set_auth(self):
        if self.apikey is None:
            # TODO: raise error
            return False
        self._session.auth = (self.apikey, 'dummy')
        return True


class SmartZoneBase(BaseAPI):
    def __init__(self, base_url, username, password, verify_ssl=True):
        self.base_url = base_url
        self.username = username
        self.password = password
        self._session = requests.Session()
        self.verify_ssl = verify_ssl
        self._session.verify = verify_ssl
        self.login()

    def logout(self):
        '''
        Use this API command to log off of the controller.
        '''
        r = self.session.delete(self.endpoint('v8_2/session'))
        return super().logout()


    def endpoint(self, path):
        url = urljoin(self.base_url, '/wsg/api/public/')
        return urljoin(url, path)

    def filters_json(self, filter_type, filter_value, start, end, search_type=None, search_value=None, attributes=None):
        if search_type is None:
            search_type = "AND"
        if search_value is None:
            search_value = ""
        if attributes is None:
            attributes = "*"
        json = {
                 "filters": [
                   {
                     "type": filter_type,
                     "value": filter_value
                   }
                 ],
                "extraTimeRange": {
                    "start": start,
                    "end": end,
                    "interval": 90000
                    },
                  "fullTextSearch": {
                    "type": search_type,
                    "value": search_value
                 },
                 "attributes": [
                    attributes
                 ]
                }
        return json



import requests
from datetime import datetime, timedelta
from urllib.parse import urljoin

from .base import SmartZoneBase


class SmartZoneClient(SmartZoneBase):
    '''
    Documentation: http://docs.ruckuswireless.com/smartzone/5.1.2/vsze-public-api-reference-guide-512.html#api-information

    Controller: https://n01.ruckus.cloud.mambowifi.com:8443
    '''
    ###### Convert DATETIME to TIMESTAMP mileseconds
    def ts_format(self, d):
        '''
            Convert to UNIX timestamp in ms
        '''
        return int(d.timestamp()*1000)

    def datetemp(self, date_range):
        a = int(date_range[0].timestamp()*1000)
        b = int(date_range[1].timestamp()*1000)
        if (a > 0):
            a += 1000
        if (b > 0):
            b += 1000
        date_range = (a, b)
        return date_range

########## LOGON SESSION
    def login(self):
        '''
        Use this API command to log on to the controller and acquire a valid logon session.
        '''
        url = self.endpoint('v8_2/session')
        json = {
                'username': self.username,
                'password': self.password
                }
        r = self.session.post(url, json=json)
        return print(r)

    def retrieve(self):
        '''
        Use this API command to retrieve information about the current logon session.
        '''
        url = self.endpoint('v8_2/session')
        r = self.session.get(url)
        # validação
        return r.json()

########## SESSION MANAGEMENT
    def sessionManagement(self):
        '''
        Use this API command to retrieve information about the current logon sessions.
        '''
        url = self.endpoint('v8_2/sessionManagement')
        r = self.session.get(url)
        # validação
        return r.json()

########## AP GROUP
    def apGroup_retrieve(self, ID, zoneId):
        '''
        Required: id and zoneid (Variable 'id' is 'ID' in this function)
        Use this API command to retrieve information about an AP group.
        '''
        url = self.endpoint('v8_2/rkszones/{}/apgroups/{}'.format(zoneId, ID))
        r = self.session.get(url)
        # validação
        return r.json()

    def apGroup_retrieveList(self, zoneId, index=None, listSize=None):
        '''
        Use this API command to retrieve the list of AP groups that belong to a zone.
        -
        Required: zoneid
        index: string (optional) - The index of the first entry to be retrieved. Default: 0
        listSize: string (optional) - The maximum number of entries to be retrieved. Default: 100
        '''
        data = {}
        if index is None:
            data['index'] = '0'
        else:
            data['index'] = index
        if listSize is None:
            data['listSize'] = '100'
        else:
            data['listSize'] = listSize
        url = self.endpoint('v8_2/rkszones/{}/apgroups'.format(zoneId))
        r = self.session.get(url, data=data)
        # validação
        return r.json()

    def accessPointConfiguration_retrieve(self, apMac):
        '''
        Use this API command to retrieve the configuration of an AP.
        -
        Required: apMac
        '''
        url = self.endpoint('v8_2/aps/{}'.format(apMac))
        r = self.session.get(url)
        # validação
        return r.json()

    def accessPointConfiguration_retrieveList(self, index=None, listSize=None, zoneId=None, domainId=None):
        '''
        Use this API command to retrieve the list of APs that belong to a zone or a domain.
        -
        Required: None
        index: string (optional) - The index of the first entry to be retrieved. Default: 0
        listSize: string (optional) - The maximum number of entries to be retrieved. Default: 100
        zoneId: string (optional) - Filter AP list by zone
        domainId: string (optional) - Filter AP list by domain. Default: current logon domain
        '''
        data = {}
        if index is None:
            data['index'] = '0'
        else:
            data['index'] = index
        if listSize is None:
            data['listSize'] = '100'
        else:
            data['listSize'] = listSize
        if zoneId is not None:
            data['zoneId'] = zoneId
        if domainId is not None:
            data['domainId'] = domainId
        url = self.endpoint('v8_2/aps')
        r = self.session.get(url, data=data)
        # validação
        return r.json()

    def accessPointOperation_retrieveOperationInformation(self, apMac):
        '''
        This API provide detailed AP status and configuration, therefore it was designed for single AP information retrieving. If you need to retrieve large number of ap states, please use “POST://query/ap” (refer to the “Query APs” section of the category “Access Point Operational”).
        -
        Required: apMac
        '''
        url = self.endpoint('v8_2/aps/{}/operational/summary'.format(apMac))
        r = self.session.get(url)
        # validação
        return r.json()

    def accessPointOperational_apPacketCapture(self, apMac):
        '''
        Required: apMac
        Use this API to get AP packet capture status
        '''
        url = self.endpoint('v8_2/aps/{}/apPacketCapture'.format(apMac))
        r = self.session.get(url)
        # validação
        return r.json()

    def accessPointOperational_RetrieveIndoormapList(self, filter_type, filter_value, date_range, search_type=None, search_value=None, attributes=None):
        '''
        Query indoorMap with specified filters. Please click the link for the usage of Query Criteria.
        Link: http://docs.ruckuswireless.com/smartzone/5.1.2/QueryCriteria.html
        -
        Required: filter_type and filter_value.
        Example:
        {
          "filters": [
            {
              "type": "DOMAIN",
              "value": "8b2081d5-9662-40d9-a3db-2a3cf4dde3f7"
            }
          ],
          "fullTextSearch": {
            "type": "AND",
            "value": ""
          },
          "attributes": [
            "*"
          ]
        }
        '''
        date_range = self.datetemp(date_range)
        json = self.filters_json(filter_type, filter_value, date_range[0], date_range[1], search_type, search_value, attributes)
        url = self.endpoint('v8_2/query/indoorMap')
        r = self.session.post(url, json=json)
        # validação
        return r.json()

    def accessPointOperational_queryAps(self, filter_type, filter_value, date_range, search_type=None, search_value=None, attributes=None):
        '''
        Query APs with specified filters Please click the link for the usage of Query Criteria.
        Link: http://docs.ruckuswireless.com/smartzone/5.1.2/QueryCriteria.html
        -
        Required: filter_type and filter_value.
        Example:
        {
          "filters": [
            {
              "type": "DOMAIN",
              "value": "8b2081d5-9662-40d9-a3db-2a3cf4dde3f7"
            }
          ],
          "fullTextSearch": {
            "type": "AND",
            "value": ""
          },
          "attributes": [
            "*"
          ]
        }
        '''
        date_range = self.datetemp(date_range)
        json = self.filters_json(filter_type, filter_value, date_range[0], date_range[1], search_type, search_value, attributes)
        url = self.endpoint('v8_2/query/ap')
        r = self.session.post(url, json=json)
        # validação
        return r.json()

########## WLAN GROUP
    def wlanGroup_retrieveList(self, zoneId, index=None, listSize=None):
        '''
        Use this API command to retrieve the list of WLAN groups within a zone.
        -
        Required: zoneId
        index: string (optional) - The index of the first entry to be retrieved. Default: 0
        listSize: string (optional) - The maximum number of entries to be retrieved. Default: 100
        '''
        url = self.endpoint('v8_2/rkszones/{}/wlangroups'.format(zoneId))
        data = {}
        if index is None:
            data['index'] = '0'
        else:
            data['index'] = index
        if listSize is None:
            data['listSize'] = '100'
        else:
            data['listSize'] = listSize
        r = self.session.get(url, data = data)
        # validação
        return r.json()

########## WLAN
    def wlan_retrieveList(self, zoneId, index=None, listSize=None):
        '''
        Use this API command to retrieve a list of WLANs within a zone.
        -
        Required: zoneId
        index: string (optional) - The index of the first entry to be retrieved. Default: 0
        listSize: string (optional) - The maximum number of entries to be retrieved. Default: 100
        '''
        url = self.endpoint('v8_2/rkszones/{}/wlans'.format(zoneId))
        data = {}
        if index is None:
            data['index'] = '0'
        else:
            data['index'] = index
        if listSize is None:
            data['listSize'] = '100'
        else:
            data['listSize'] = listSize
        r = self.session.get(url, data = data)
        # validação
        return r.json()

    def wlan_queryWlans(self, filter_type, filter_value, date_range, search_type=None, search_value=None, attributes=None):
        '''
        Query WLANs with specified filters. Please click the link for the usage of Query Criteria.
        Link: http://docs.ruckuswireless.com/smartzone/5.1.2/QueryCriteria.html
        -
        Required: filter_type and filter_value
        Example:
        {
          "filters": [
            {
              "type": "DOMAIN",
              "value": "8b2081d5-9662-40d9-a3db-2a3cf4dde3f7"
            }
          ],
          "fullTextSearch": {
            "type": "AND",
            "value": ""
          },
          "attributes": [
            "*"
          ]
        }
        '''
        date_range = self.datetemp(date_range)
        json = self.filters_json(filter_type, filter_value, date_range[0], date_range[1], search_type, search_value, attributes)
        url = self.endpoint('v8_2/query/wlan')
        r = self.session.post(url, json=json)
        # validação
        return r.json()

########## ACCESS POINT APP - FALTA VALIDAR**************************
    def accessPointAPP_retrieveTotalApCount(self, zoneId=None, domainId=None):
        '''
        Use this API command to retrieve the total AP count within a zone or a domain.
        -
        Required: zoneId or domainId
        zoneId: string (optional) - Filter AP total count by zone. Default: current logon domain
        domainId: string (optional) - Filter AP total count by domain. Default: current logon domain
        '''
        assert zoneId or domainId, "ERRO: zoneId or domainId is required."
        data = {}
        if zoneId is not None:
            data['zoneId'] =  zoneId
        if domainId is not None:
            data['domainId'] = domainId
        url = self.endpoint('v8_2/aps/totalCount')
        r = self.session.get(url, data=data)
        # validação
        return r.json()


########## WIRELESS CLIENT
    def wirelessClient_retrieveTotalClientCount(self, apMac):
        '''
        Required: apMac
        Use this API command to retrieve the total client count per AP.
        '''
        url = self.endpoint('v8_2/aps/{}/operational/client/totalCount'.format(apMac))
        r = self.session.get(url)
        # validação
        return r.json()

    def wirelessClient_queryClients(self, filter_type, filter_value, date_range, search_type=None, search_value=None, attributes=None):
        '''
        Query clients with specified filters. Please click the link for the usage of Query Criteria.
        Link: http://docs.ruckuswireless.com/smartzone/5.1.2/QueryCriteria.html
        -
        Required: filter_type, filter_value
        Example:
        {
          "filters": [
            {
              "type": "DOMAIN",
              "value": "8b2081d5-9662-40d9-a3db-2a3cf4dde3f7"
            }
          ],
          "fullTextSearch": {
            "type": "AND",
            "value": ""
          },
          "attributes": [
            "*"
          ]
        }
        '''
        date_range = self.datetemp(date_range)
        json = self.filters_json(filter_type, filter_value, date_range[0], date_range[1], search_type, search_value, attributes)
        url = self.endpoint('v8_2/query/client')
        r = self.session.post(url, json=json)
        # validação
        return r.json()

    def wirelessClient_historicalClient_gen(self, filter_type, filter_value, date_range):
        has_more = True
        page = 1
        while has_more:
            d = self.wirelessClient_historicalClient(
                    filter_type,
                    filter_value,
                    date_range,
                    page=page,
                    limit=1000)
            has_more = d.get('hasMore', False)
            page += 1
            for el in d.get('list', []):
                yield el

    def wirelessClient_historicalClient(self, filter_type, filter_value, date_range, page=1, limit=1000):
        '''
        Use this API command to retrive historical client. Please click the link for the usage of Query Criteria.
        Link: http://docs.ruckuswireless.com/smartzone/5.1.2/QueryCriteria.html
        -
        Required: filter_type and filter_value.
        Example:
        {
          "filters": [
            {
              "type": "DOMAIN",
              "value": "d0d495e1-de50-40e3-8d09-e4bbeb4b4722"
            }
          ]
        }
        '''
        json = {
                "filters": [
                    {
                    "type": filter_type,
                    "value": filter_value
                    }
                    ],
                "extraTimeRange": {
                    "start": self.ts_format(date_range[0]),
                    "end": self.ts_format(date_range[1]),
                    "interval": 0
                    },
                "page": page,
                "limit": limit,
                }
        url = self.endpoint('v8_2/query/historicalclient')
        r = self.session.post(url, json=json)
        # validação
        return r.json()

    def wirelessClient_disconnectClient(self, mac, apMac):
        '''
        Use this API command to disconnect client.
        -
        Required: mac and apMac
        Example:
        {
          "mac": "E8:99:C4:AD:7C:38",
          "apMac": "C0:8A:DE:24:FA:00"

        '''
        json = {
                    "mac": mac,
                    "apMac": apMac
                }
        url = self.endpoint('v8_2/clients/disconnect')
        r = self.session.post(url, json=json)
        # validação
        return r.json()

    def trafficAnalysis_clientUsageWlan (self, filter_type, filter_value, date_range, interval='86400000', rate='tx+rx', frequence='2.4G+5G', page=1, limit=1000):
        '''clientUsageWlan return number total of clients in zone, name of device and total bytes consumed
           Params: (self, filter_type, filter_value, date_range, interval='90000', rate='tx+rx', frequence='2.4G+5G', page=1, limit=1000)
           Required: filter_type, filter_value and date_range
           Optional: interval, rate, frequence, page and limit

           filter_type & filter_value:
                --------------------------------------------------------------------------
               |TYPE(type)|    Value    |    Example(value)                               |
               |--------------------------------------------------------------------------|
               |AP        |apMac        |{AP, 11:22:33:44:55:66}                          |
               |APGROUP   |apGroupId    |{APGROUP, 758b6970-032a-11e7-9e78-0a0027000000}  |
               |CLIENT    |clientMac    |{CLIENT, AB:CD:00:00:00:03}                      |
               |DOMAIN    |domainId     |{DOMAIN, 8b2081d5-9662-40d9-a3db-2a3cf4dde3f7}   |
               |INDOORMAP |indoorMapId  |{INDOORMAP, 08733520-0a32-11e7-89a3-0a0027000000}|
               |WLAN      |wlanId       |{WLAN, 1}                                        |
               |ZONE      |zoneId       |{ZONE, 91fa3fe0-03da-11e7-8d82-0a0027000000}     |
                --------------------------------------------------------------------------
            frequence: 2.4G, 5G or 2.4G+5G
            rate: tx, rx or tx+rx
            interval: interval in mileseconds
            limit: num of limite itens on page
            page: number os pages
        '''
        date_range = self.datetemp(date_range)
        json ={
                "filters":[{
                    "type": filter_type,
                    "value": filter_value
                }],
                "extraFilters":[{
                    "type":"RADIOID",
                    "value": frequence
                    }],
                "extraNotFilters":[{
                    "type":"MONITORINGENABLED",
                    "value":"true"
                    }],
                "attributes":[
                    rate,
                    "Host_Name"
                    ],
                "extraTimeRange":{
                    "start": date_range[0],
                    "end": date_range[1],
                    "interval": interval},
                "options":{
                    },
                "limit": limit,
                "page":page}
        url = self.endpoint('v8_2/trafficAnalysis/client/usage/wlan')
        r = self.session.post(url, json=json)
        return r.json()



from .smartzone_api import SmartZoneClient
from .vspot_api import VSpotClient

from . import smartzone_api, vspot_api

import urllib3
urllib3.disable_warnings()

__all__ = ['SmartZoneClient', 'VSpotClient']



from .base import VSpotBase
from .decorators import returns_json, returns_gen, retry_on_processing, throw_for
from .exceptions import WaitForProcess, Unauthorized


DATE_FORMAT = '%Y-%m-%d'

class VSpotClient(VSpotBase):
    '''
    Documentation: https://18.228.238.187/api_explorer
    '''

    def email_login(self, email, password):
        d = self.api_keys(email, password)
        self.apikey = d.get('api_key')
        self._set_auth()
        self.role = d.get('role')
        return d

    @returns_json
    @throw_for(401, Unauthorized)
    def api_keys(self, email, password):
        data = {
                'email': email,
                'password': password,
                }
        return self.post(self.endpoint('api_keys.json'), data=data)

##### VENUES #####

    @returns_json
    def venues(self):
        '''
        Return JSON containing a list of all location IDs
        -------------------------------
        Name                   | Type
        -------------------------------
        venue_id               | string
        name                   | string
        exterior_thumbnail_url | string
        Example:
        [{
            'venue_id': 'mambo01',
            'name': 'mambo01',
            'exterior_thumbnail_url': 'https://18.228.238.187/venues/mambo01/files/exterior_thumbnail.png'
        }]
        '''
        return self.get(self.endpoint('venues.json'))

    @returns_json
    def venue(self, venue_id):
        '''
        Return JSON containing a data for one venue_id
        -
        Required: venue_id
        -----------------------------
        Name               | Type
        -----------------------------
        venue_id           | string
        name               | string
        exterior_image_url | string
        street_address     | string
        locality           | string
        region             | string
        postal_code        | string
        country_name       | string
        coordinates        | array
        ---|0              | position
           ---|time_zone_id| string
        -----------------------------
        Example:
        {
            'venue_id': 'mambo01',
             'name': 'mambo01',
             'exterior_image_url': 'https://18.228.238.187/venues/mambo01/files/exterior_image.png',
             'street_address': 'Rua São José ',
             'locality': 'Rio de Janerio',
             'region': 'Rio de Janeiro',
             'postal_code': '20010-020',
             'country_name': 'Brazil',
             'coordinates': [-22.906079, -43.17678],
             'time_zone_id': 'America/Sao_Paulo'
        }
        '''
        return self.get(self.endpoint('venues/{}.json', venue_id))

##### RADIO MAPS #####
    @returns_json
    def radio_maps(self, venue_id):
        '''
        Return radio map
        -
        Required: venue_id
        -------------------------
        Name            | Type
        -------------------------
        name            | string
        production      | boolean
        start_timestamp | string
        end_timestamp   | string
        -------------------------
        Example:
        [{
            'name': '1_0_0',
            'production': True,
            'start_timestamp': '2020-01-07T00:00:00-02:00',
            'end_timestamp': None
        }]

        '''
        return self.get(self.endpoint('venues/{}/radio_maps.json', venue_id))

    @returns_json
    def radio_map(self, venue_id, name):
        '''
        REQUIRED: venue_id and name
        -------------------------
        Name            | Type
        -------------------------
        name            | string
        production      | boolean
        start_timestamp | string
        end_timestamp   | string
        -------------------------
        Example:
        {
            'name': '1_0_0',
             'production': True,
             'start_timestamp': '2020-01-07T00:00:00-02:00',
             'end_timestamp': None,
             'width': 1889.0,
             'height': 856.0,
             'scale': 10,
             'floors': [{'number': 1,
               'display_name': 'Andar Mambo WiFi',
               'map_image_url': 'https://18.228.238.187/venues/mambo01/radio_maps/1_0_0/files/floor_1.png',
               'inside_image_url': 'https://18.228.238.187/venues/mambo01/radio_maps/1_0_0/files/floor_1_inside.png',
               'zone_maps': [
                            ]
                        }]
        }
        '''
        return self.get(self.endpoint('venues/{}/radio_maps/{}.json', venue_id, name))

##### LOCATIONS #####
    @returns_gen
    def locations_by_date(self, venue_id, date, macs=[], limit=''):
        '''
        REQUIRED: venue_id and date
        -----------------------------------------------------
        Name            | Example                   | Type
        -----------------------------------------------------
        mac             | B40C00000000              | String
        timestamp       | 2020-01-16T13:04:38-02:00 | String
        floor_number    | 0                         | Int
        x               | 0.0                       | Int
        y               | 0.0                       | Int
        located_inside  | False                     | Boolean
        zones           | []                        | Array
        -{zone_map_name |                           | String
          zone_name  }  |                           | String
        -----------------------------------------------------
        Example:
        {
            'mac': 'B40C00000000',
            'timestamp': '2020-01-16T13:04:38-02:00',
            'floor_number': 0,
            'x': 0.0,
            'y': 0.0,
            'located_inside': False,
            'zones': []
        }
        '''

        data = {
            'date': date.strftime(DATE_FORMAT),
            'limit': limit,
            'macs[]': macs
        }
        r = self.get(self.endpoint('venues/{}/locations/by_date.json', venue_id), params=data)
        return self.link_gen(r)

    @returns_json
    def locations_last_known(self, venue_id, seconds_ago=60, macs=[]):
        '''
        REQUIRED: venue_id
        -----------------------------------------------------
        Name            | Example                   | Type
        -----------------------------------------------------
        mac             | B40C00000000              | String
        timestamp       | 2020-01-16T13:04:38-02:00 | String
        floor_number    | 0                         | Int
        x               | 0.0                       | Int
        y               | 0.0                       | Int
        located_inside  | False                     | Boolean
        zones           | []                        | Array
        -{zone_map_name |                           | String
          zone_name  }  |                           | String
        -----------------------------------------------------
        Example:
        {
            'mac': 'B40C00000000',
            'timestamp': '2020-01-16T13:04:38-02:00',
            'floor_number': 0,
            'x': 0.0,
            'y': 0.0,
            'located_inside': False,
            'zones': []
        }
        '''
        data = {
            'macs[]': macs,
            'seconds_ago': seconds_ago
        }
        return self.get(self.endpoint('venues/{}/locations/last_known.json', venue_id), params=data)

##### METRICS #####
    @returns_json
    @retry_on_processing
    @throw_for(202, WaitForProcess)
    def metrics_impressions(self, venue_id, start_date, end_date, granularity='daily', floor_number='', zone_map_name='', zone_name=''):
        '''
        Required: venue_id, start_date and end_date
        ---------------------------------------------
        Name            |   Format           | Type
        ---------------------------------------------
        venue_id        |   mambo01          | String
        start_date      |   YYYY-MM-DD       | String
        end_date        |   YYYY-MM-DD       | String
        granularity     |   daily or hourly  | String
        floor_number    |   0                | Int
        zone_map_name   |                    | String
        zone_name       |                    | String
        ---------------------------------------------
        Example:
        [
          {
            "timestamp": "string",
            "impressions": [
              {
                "x": 0,
                "y": 0,
                "count": 0,
                "located_inside": 0,
                "located_outside": 0
              }
            ]
          }
        ]
        '''
        assert start_date < end_date, 'Start date should be before end date'
        if floor_number or zone_map_name or zone_name:
            assert floor_number and zone_map_name and zone_name, 'floor_number, zone_map_name and zone_name must be used together'
        data = {
                'start_date': start_date.strftime(DATE_FORMAT),
                'end_date': end_date.strftime(DATE_FORMAT),
                'granularity': granularity,
                'floor_number': floor_number,
                'zone_map_name': zone_map_name,
                'zone_name': zone_name,
                }
        return self.get(self.endpoint('venues/{}/metrics/impressions.json', venue_id), params=data)

    @returns_json
    @retry_on_processing
    @throw_for(202, WaitForProcess)
    def metrics_unique_visitors(self, venue_id, start_date, end_date, granularity='daily', floor_number='', zone_map_name='', zone_name='', list_macs=False):
        '''
        Required: venue_id, start_date and end_date
        ---------------------------------------------
        Name            |   Format           | Type
        ---------------------------------------------
        venue_id        |   mambo01          | String
        start_date      |   YYYY-MM-DD       | String
        end_date        |   YYYY-MM-DD       | String
        granularity     |   daily or hourly  | String
        floor_number    |   0                | Int
        zone_map_name   |                    | String
        zone_name       |                    | String
        list_macs       |   true or false    | String
        ---------------------------------------------
        Example:
        [
          {
            "timestamp": "string",
            "unique_visitors": 0,
            "unique_macs": [
              "string"
            ]
          }
        ]
        '''
        assert start_date < end_date, 'Start date should be before end date'
        if floor_number or zone_map_name or zone_name:
            assert floor_number and zone_map_name and zone_name, 'floor_number, zone_map_name and zone_name must be used together'
        data = {
                'start_date': start_date.strftime(DATE_FORMAT),
                'end_date': end_date.strftime(DATE_FORMAT),
                'granularity': granularity,
                'list_macs': list_macs,
                'floor_number': floor_number,
                'zone_map_name': zone_map_name,
                'zone_name': zone_name,
                }
        return self.get(self.endpoint('venues/{}/metrics/unique_visitors.json', venue_id), params=data)

    @returns_json
    @retry_on_processing
    @throw_for(202, WaitForProcess)
    def metrics_average_dwell_time(self, venue_id, start_date, end_date, floor_number='', zone_map_name='', zone_name=''):
        '''
        Required: venue_id, start_date and end_date
        ---------------------------------------------
        Name            |   Format           | Type
        ---------------------------------------------
        venue_id        |   mambo01          | String
        start_date      |   YYYY-MM-DD       | String
        end_date        |   YYYY-MM-DD       | String
        floor_number    |   0                | Int
        zone_map_name   |                    | String
        zone_name       |                    | String
        ---------------------------------------------
        Example:
        [
          {
            "timestamp": "string",
            "average_dwell_time": 0
          }
        ]
        '''
        assert start_date < end_date, 'Start date should be before end date'
        if floor_number or zone_map_name or zone_name:
            assert floor_number and zone_map_name and zone_name, 'floor_number, zone_map_name and zone_name must be used together'
        data = {
                'start_date': start_date.strftime(DATE_FORMAT),
                'end_date': end_date.strftime(DATE_FORMAT),
                'floor_number': floor_number,
                'zone_map_name': zone_map_name,
                'zone_name': zone_name,
                }
        return self.get(self.endpoint('venues/{}/metrics/average_dwell_time.json', venue_id), params=data)

    @returns_json
    @retry_on_processing
    @throw_for(202, WaitForProcess)
    def metrics_dwell_time_distribution(self, venue_id, start_date, end_date, floor_number='', zone_map_name='', zone_name=''):
        '''
        Required: venue_id, start_date and end_date
        ---------------------------------------------
        Name            |   Format           | Type
        ---------------------------------------------
        venue_id        |   mambo01          | String
        start_date      |   YYYY-MM-DD       | String
        end_date        |   YYYY-MM-DD       | String
        floor_number    |   0                | Int
        zone_map_name   |                    | String
        zone_name       |                    | String
        ---------------------------------------------
        Example:
        [
          {
            "timestamp": "string",
            "dwell_time_distribution": {
              "1-5 mins": 0,
              "6-10 mins": 0,
              "11-30 mins": 0,
              "31-60 mins": 0,
              "> 60 mins": 0
            }
          }
        ]
        '''
        assert start_date < end_date, 'Start date should be before end date'
        if floor_number or zone_map_name or zone_name:
            assert floor_number and zone_map_name and zone_name, 'floor_number, zone_map_name and zone_name must be used together'
        data = {
                'start_date': start_date.strftime(DATE_FORMAT),
                'end_date': end_date.strftime(DATE_FORMAT),
                'floor_number': floor_number,
                'zone_map_name': zone_map_name,
                'zone_name': zone_name,
                }
        return self.get(self.endpoint('venues/{}/metrics/dwell_time_distribution.json', venue_id), params=data)

    @returns_json
    @retry_on_processing
    @throw_for(202, WaitForProcess)
    def metrics_new_vs_repeat(self, venue_id, start_date, end_date, floor_number='', zone_map_name='', zone_name=''):
        '''
        Required: venue_id, start_date and end_date
        ---------------------------------------------
        Name            |   Format           | Type
        ---------------------------------------------
        venue_id        |   mambo01          | String
        start_date      |   YYYY-MM-DD       | String
        end_date        |   YYYY-MM-DD       | String
        floor_number    |   0                | Int
        zone_map_name   |                    | String
        zone_name       |                    | String
        ---------------------------------------------
        Example:
        [
          {
            "timestamp": "string",
            "new_vs_repeat": {
              "New": 0,
              "Repeat": 0
            }
          }
        ]
        '''
        assert start_date < end_date, 'Start date should be before end date'
        if floor_number or zone_map_name or zone_name:
            assert floor_number and zone_map_name and zone_name, 'floor_number, zone_map_name and zone_name must be used together'
        data = {
                'start_date': start_date.strftime(DATE_FORMAT),
                'end_date': end_date.strftime(DATE_FORMAT),
                'floor_number': floor_number,
                'zone_map_name': zone_map_name,
                'zone_name': zone_name,
                }
        return self.get(self.endpoint('venues/{}/metrics/new_vs_repeat.json', venue_id), params=data)

    @returns_json
    @retry_on_processing
    @throw_for(202, WaitForProcess)
    def metrics_repeat_count(self, venue_id, start_date, end_date, floor_number='', zone_map_name='', zone_name=''):
        '''
        Required: venue_id, start_date and end_date
        ---------------------------------------------
        Name            |   Format           | Type
        ---------------------------------------------
        venue_id        |   mambo01          | String
        start_date      |   YYYY-MM-DD       | String
        end_date        |   YYYY-MM-DD       | String
        floor_number    |   0                | Int
        zone_map_name   |                    | String
        zone_name       |                    | String
        ---------------------------------------------
        Example:
        [
          {
            "timestamp": "string",
            "repeat_count": {
              "1 time": 0,
              "2-5 times": 0,
              "6-10 times": 0,
              "> 10 times": 0
            }
          }
        ]
        '''
        assert start_date < end_date, 'Start date should be before end date'
        if floor_number or zone_map_name or zone_name:
            assert floor_number and zone_map_name and zone_name, 'floor_number, zone_map_name and zone_name must be used together'
        data = {
                'start_date': start_date.strftime(DATE_FORMAT),
                'end_date': end_date.strftime(DATE_FORMAT),
                'floor_number': floor_number,
                'zone_map_name': zone_map_name,
                'zone_name': zone_name,
                }
        return self.get(self.endpoint('venues/{}/metrics/repeat_count.json', venue_id), params=data)

    @returns_json
    @retry_on_processing
    @throw_for(202, WaitForProcess)
    def metrics_days_since_last_visit(self, venue_id, start_date, end_date, floor_number='', zone_map_name='', zone_name=''):
        '''
        Required: venue_id, start_date and end_date
        ---------------------------------------------
        Name            |   Format           | Type
        ---------------------------------------------
        venue_id        |   mambo01          | String
        start_date      |   YYYY-MM-DD       | String
        end_date        |   YYYY-MM-DD       | String
        floor_number    |   0                | Int
        zone_map_name   |                    | String
        zone_name       |                    | String
        ---------------------------------------------
        Example:
        [
          {
            "timestamp": "string",
            "days_since_last_visit": {
              "1 day": 0,
              "2-7 days": 0,
              "8-30 days": 0,
              "> 30 days": 0
            }
          }
        ]
        '''
        assert start_date < end_date, 'Start date should be before end date'
        if floor_number or zone_map_name or zone_name:
            assert floor_number and zone_map_name and zone_name, 'floor_number, zone_map_name and zone_name must be used together'
        data = {
                'start_date': start_date.strftime(DATE_FORMAT),
                'end_date': end_date.strftime(DATE_FORMAT),
                'floor_number': floor_number,
                'zone_map_name': zone_map_name,
                'zone_name': zone_name,
                }
        return self.get(self.endpoint('venues/{}/metrics/days_since_last_visit.json', venue_id), params=data)

##### DEVICES

    def devices_find(self, venue_id, ip):
        '''
        Required: venue_id and ip
        --------------------------
        Name        | Type
        --------------------------
        mac         | String
        ip          | String
        Example:
        {
            "mac": "string",
            "ip": "string"
        }
        '''
        r = self.get(self.endpoint('venues/{}/devices/{}/find.json', venue_id, ip))
        return r.json()

    def devices_ip(self, venue_id, ip):
        '''
        Required: venue_id and ip
        --------------------------
        Name        | Type
        --------------------------
        mac         | String
        ip          | String
        Example:
        {
            "mac": "string",
            "ip": "string"
        }
        '''
        r = self.get(self.endpoint('venues/{}/devices/{}.json', venue_id, ip))
        return r.json()

##### beta

    @returns_json
    @retry_on_processing
    @throw_for(202, WaitForProcess)
    def metrics_consolidated(self, venue_id, start_date, end_date, floor_number='', zone_map_name='', zone_name=''):
        assert start_date < end_date, 'Start date should be before end date'
        if floor_number or zone_map_name or zone_name:
            assert floor_number and zone_map_name and zone_name, 'floor_number, zone_map_name and zone_name must be used together'

        data = {
                'start_date': start_date.strftime(DATE_FORMAT),
                'end_date': end_date.strftime(DATE_FORMAT),
                'floor_number': floor_number,
                'zone_map_name': zone_map_name,
                'zone_name': zone_name,
                }

        return self.get(self.endpoint('venues/{}/metrics/consolidated.json', venue_id), params=data)

    @returns_json
    def metrics_time_divisions(self, venue_id, start_date, end_date, granularity='daily'):
        data = {
                'start_date': start_date.strftime(DATE_FORMAT),
                'end_date': end_date.strftime(DATE_FORMAT),
                'granularity': granularity,
                }
        return self.get(self.endpoint('venues/{}/metrics/time_divisions.json', venue_id), params=data)

    @returns_json
    @retry_on_processing
    @throw_for(202, WaitForProcess)
    def metrics_heatmap(self, venue_id, start_date, end_date, time_division, granularity='daily', floor_number='', zone_map_name='', zone_name=''):
        data = {
                'start_date': start_date.strftime(DATE_FORMAT),
                'end_date': end_date.strftime(DATE_FORMAT),
                'granularity': granularity,
                'time_division': time_division,
                'floor_number': floor_number,
                'zone_map_name': zone_map_name,
                'zone_name': zone_name,
                }

        return self.get(self.endpoint('venues/{}/metrics/heatmap.json', venue_id), params=data)

    @returns_gen
    def metrics_heatmap_range(self, venue_id, start_date, end_date, granularity='daily', floor_number='', zone_map_name='', zone_name=''):
        time_divisions = self.metrics_time_divisions(venue_id, start_date, end_date, granularity)

        return ( self.metrics_heatmap(
                    venue_id,
                    start_date,
                    end_date,
                    t,
                    granularity,
                    floor_number,
                    zone_map_name,
                    zone_name)
                 for t in time_divisions
                )

    @returns_json
    def metrics_realtime_heatmap(self, venue_id, floor_number='', zone_map_name='', zone_name=''):
        '''
        {
            "timestamp": timestamp_iso,
            "heatmap_data": [
                {
                    "floor_number": int,
                    "floor_name": str,
                    "floor_map_url": url,
                    "floor_width": float,
                    "floor_height": float,
                    "floor_scale": int,
                    "located_inside": int,
                    "located_outside": int,
                    "floor_data": [
                        {
                            "x": float,
                            "y": float,
                            "count": int
                        }
                    ]
                }
            ]
        }
        '''
        if floor_number or zone_map_name or zone_name:
            assert floor_number and zone_map_name and zone_name, 'floor_number, zone_map_name and zone_name must be used together'

        data = {
            'floor_number': floor_number,
            'zone_map_name': zone_map_name,
            'zone_name': zone_name,
        }
        return self.get(self.endpoint('venues/{}/metrics/real_time_heatmap.json', venue_id), params=data)



# coding: utf-8

from setuptools import setup, find_packages

setup(
    name = "ruckus-python-api",
    version = "0.0.10",
    packages = find_packages('src'),
    package_dir = {"": "src"},
    install_requires = [
        'requests>=2.21.0,<3',
    ],
    author='Nelson Fonseca',
    author_email='nelson@mambowifi.com',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Information Technology',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
    ],
)
